/*
 * ##################################################
 * #                                                #
 * # rkNumberConverter                              #
 * # Number system conversation                     #
 * #                                                #
 * # Copyright (C) by René Knipschild               #
 * #                                                #
 * #                                email@rkcsd.com #
 * #                         www.reneknipschild.net #
 * #                                                #
 * ##################################################
 *
 * File: UserInterface.java - gui
 * Version: 1.0.0
 * Last modified: 2011/04/20 09:29 CEST
 * Author: rkCSD (go.reneknipschild.net/com)
 *
 * ===Notes==========================================
 * There are currently no notes.
 * ==================================================
 */

package com.rkcsd.apps.desktoptools.numberconverter.gui;

import java.awt.BorderLayout;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Toolkit;
import java.awt.datatransfer.StringSelection;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowFocusListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.WindowConstants;

import com.rkcsd.apps.desktoptools.numberconverter.tools.Browser;
import com.rkcsd.apps.desktoptools.numberconverter.tools.NumberConverter;

public class UserInterface {
	private JFrame window;

	private JPanel frame;

	private JComboBox inputSystem;

	private JTextField input;

	private JComboBox outputSystem;

	private JTextField output;

	private JCheckBox clipboard;

	private JButton confirm;
	
	private JLabel copyright;

	public UserInterface() {
		this.window = new JFrame("rkNumberConverter");
		this.window.setIconImage(Toolkit.getDefaultToolkit().getImage(ClassLoader.getSystemResource("com/rkcsd/apps/desktoptools/numberconverter/resources/rkIcon.png")));
		this.window.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
		this.frame = new JPanel();
		this.frame.setBorder(BorderFactory.createEmptyBorder(4, 4, 4, 4));
		ComfortGridBagLayout layout = new ComfortGridBagLayout();
		this.frame.setLayout(layout);

		this.inputSystem = new JComboBox(NumberConverter.NUMBERSYSTEMS);
		this.input = new JTextField();
		this.outputSystem = new JComboBox(NumberConverter.NUMBERSYSTEMS);
		this.output = new JTextField();
		this.clipboard = new JCheckBox("Copy result into clipboard");
		this.confirm = new JButton("Convert");
		this.copyright = new JLabel("<html><p>Copyright © 2009 René Knipschild (<a href=\"http://www.reneknipschild.net/\">www.reneknipschild.net</a>)</p></html>)");

		this.window.addWindowFocusListener(new WindowFocusListener() {
			public void windowGainedFocus(WindowEvent event) {
				UserInterface.this.input.requestFocusInWindow();
			}

			public void windowLostFocus(WindowEvent event) {

			}
		});
		this.inputSystem.setSelectedIndex(2);
		this.outputSystem.setSelectedIndex(0);
		this.output.setEditable(false);
		this.output.setFocusable(false);
		this.confirm.setPreferredSize(new Dimension(200, this.confirm.getPreferredSize().height));
		this.confirm.setEnabled(false);
		this.copyright.setBorder(BorderFactory.createEmptyBorder());

		layout.createGridBagConstraints(this.inputSystem, 0, 0, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, 0, 0);
		layout.createGridBagConstraints(this.input, 1, 0, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 16, 0, 0, 0);
		layout.createGridBagConstraints(this.outputSystem, 0, 1, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, 0, 0);
		layout.createGridBagConstraints(this.output, 1, 1, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 16, 0, 0, 0);
		layout.createGridBagConstraints(this.clipboard, 0, 2, 2, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, 0, 0);
		layout.createGridBagConstraints(this.confirm, 0, 3, 2, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, 0, 0);
		layout.createGridBagConstraints(this.copyright, 0, 4, 2, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, 0, 0);

		this.inputSystem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				UserInterface.this.reset(0);
			}
		});

		this.input.addKeyListener(new KeyListener() {
			public void keyPressed(KeyEvent event) {
				if (event.getKeyCode() == KeyEvent.VK_ENTER) {
					UserInterface.this.run();
				}
			}

			public void keyReleased(KeyEvent event) {
				UserInterface.this.validate();
			}

			public void keyTyped(KeyEvent event) {

			}
		});

		this.outputSystem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				UserInterface.this.reset(1);
			}
		});

		this.confirm.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				UserInterface.this.run();
			}
		});

		this.copyright.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent e) {
				Browser.open("http://www.reneknipschild.net/");
			}
			public void mousePressed(MouseEvent e) {

			}
			public void mouseReleased(MouseEvent e) {
				
			}
			public void mouseEntered(MouseEvent e) {
				UserInterface.this.copyright.setCursor(new Cursor(Cursor.HAND_CURSOR));
			}
			public void mouseExited(MouseEvent e) {
				UserInterface.this.copyright.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
			}			
		});
		
		this.frame.add(this.inputSystem);
		this.frame.add(this.input);
		this.frame.add(this.outputSystem);
		this.frame.add(this.output);
		this.frame.add(this.clipboard);
		this.frame.add(this.confirm);
		this.frame.add(this.copyright);

		this.window.add(this.frame, BorderLayout.CENTER);
		this.window.pack();
		this.window.setResizable(false);
		this.window.setLocationRelativeTo(null);
		this.window.setVisible(true);
	}

	private void reset(int field) {
		if (field == 0) {
			this.input.setText("");
			this.output.setText("");
			this.confirm.setEnabled(false);
		} else if (field == 1) {
			this.output.setText("");
		}
	}

	private void validate() {
		String inputvalue = this.input.getText();
		if (inputvalue.matches(NumberConverter.NUMBERSYSTEMS_MATCH[this.inputSystem.getSelectedIndex()]) && !inputvalue.isEmpty()) {
			this.confirm.setEnabled(true);
		} else {
			this.confirm.setEnabled(false);
		}
	}

	private void run() {
		if (this.confirm.isEnabled()) {
			String result = "";
			int zero = -1;
			try {
				zero = Integer.parseInt(this.input.getText());
			} catch (Exception exception) {

			}
			if (zero == 0) {
				result = "0";
			} else {
				result = NumberConverter.convert(this.input.getText(), this.inputSystem.getSelectedIndex(), this.outputSystem.getSelectedIndex());
			}
			if (this.clipboard.isSelected()) {
				Toolkit.getDefaultToolkit().getSystemClipboard().setContents(new StringSelection(result), null);
			}
			this.output.setText(result);
		}
	}	
}